package id

import (
	"fmt"
)

const (
	Collection        = "collection"
	CollectionsPrefix = "cols"
)

var _ Descriptor = &CollectionId{}

type CollectionId struct {
	EnvironmentId
	CollectionID string `json:"col_id,omitempty" bson:"col_id, omitempty"`
}

func (id *CollectionId) New() Descriptor {
	return &CollectionId{EnvironmentId: EnvironmentId{SpaceId: SpaceId{}}}
}

func (id *CollectionId) Type() string { return Collection }

func (id *CollectionId) String() string {
	return Join(id.EnvironmentId.String(), CollectionsPrefix, id.CollectionID)
}

func (id *CollectionId) Map() map[string]any {
	m := id.EnvironmentId.Map()
	m["col_id"] = id.CollectionID
	m["type"] = Collection
	return m
}

func (id *CollectionId) FromParts(parts []string) error {
	if len(parts) != 6 || parts[4] != CollectionsPrefix {
		return ErrInvalidID
	}
	if err := id.EnvironmentId.FromParts(parts[:4]); err != nil {
		return err
	}
	id.CollectionID = parts[5]
	return nil
}

func (id *CollectionId) FromMap(m map[string]any) error {
	id.CollectionID, _ = m["col_id"].(string)
	if id.CollectionID == "" {
		return fmt.Errorf("%w: CollectionID required", ErrInvalidID)
	}
	return id.EnvironmentId.FromMap(m)
}

func (id *CollectionId) Validate() error {
	if id.CollectionID == "" {
		return fmt.Errorf("%w: CollectionID required", ErrInvalidID)
	}
	return id.EnvironmentId.Validate()
}

func NewCollectionId(spaceID, envID, id string) *ObjectId {
	return &ObjectId{Descriptor: &CollectionId{EnvironmentId: EnvironmentId{SpaceId: SpaceId{SpaceID: spaceID}, EnvironmentID: envID}, CollectionID: id}}
}
