package zap

import (
	"testing"

	"github.com/stretchr/testify/require"
	"go.uber.org/zap/zapcore"
	"go.uber.org/zap/zaptest/observer"
)

func TestWithChannel_WriteSingleChannel(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)
	core = WithChannel(core, "test")

	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test")}))
	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("empty")})) // запись не попадет в лог

	require.Equal(t, 1, logs.Len())
}

func TestWithChannel_WriteMultiplyChannels(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)

	core = zapcore.NewTee(
		WithChannel(core, "test1"),
		WithChannel(core, "test2"),
	)

	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test1", "test2")})) // запись попадет сразу в 2 core

	require.Equal(t, 2, logs.Len())
}

func TestWithDefaultChannels(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)

	core = zapcore.NewTee(
		WithChannel(core, "test1", true),
		WithChannel(core, "test2"),
	)

	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test1")}))
	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test3")})) // эта запись не попадет в лог
	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{}))

	require.Equal(t, 2, logs.Len())
}

func TestWithChannels(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)

	core = WithChannels(core, "test1", "test2")

	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test1")}))
	require.NoError(t, core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{Channels("test2")}))

	require.Equal(t, 2, logs.Len())
}
